/*--------------------------------------------------------------------------+
$Id$
|                                                                          |
| Copyright 2011 ForTISS GmbH                                              |
|                                                                          |
| Licensed under the Apache License, Version 2.0 (the "License");          |
| you may not use this file except in compliance with the License.         |
| You may obtain a copy of the License at                                  |
|                                                                          |
|    http://www.apache.org/licenses/LICENSE-2.0                            |
|                                                                          |
| Unless required by applicable law or agreed to in writing, software      |
| distributed under the License is distributed on an "AS IS" BASIS,        |
| WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. |
| See the License for the specific language governing permissions and      |
| limitations under the License.                                           |
+--------------------------------------------------------------------------*/
package org.fortiss.tooling.kernel.internal;

import java.util.List;

import org.eclipse.emf.ecore.EObject;
import org.fortiss.tooling.kernel.base.EObjectAwareServiceBase;
import org.fortiss.tooling.kernel.interfaces.ICompositionContext;
import org.fortiss.tooling.kernel.interfaces.ICompositor;
import org.fortiss.tooling.kernel.services.ICompositorService;
import org.fortiss.tooling.kernel.services.IPrototypeService.Prototype;

/**
 * This class implements the {@link ICompositorService} interface.
 * 
 * @author hoelzl
 * @author $Author$
 * @version $Rev$
 * @ConQAT.Rating YELLOW Hash: 1AB8E5D4937CE2C77649035548CA11DF
 */
public final class CompositorService extends
		EObjectAwareServiceBase<ICompositor<EObject>> implements
		ICompositorService {

	/** The compositor extension point ID. */
	private static final String EXTENSION_POINT_NAME = "org.fortiss.tooling.kernel.modelElementCompositor";

	/** The compositor configuration element name. */
	private static final String CONFIGURATION_ELEMENT_NAME = "modelElementCompositor";

	/** The compositor class attribute name. */
	private static final String HANDLER_CLASS_ATTRIBUTE_NAME = "compositor";

	/** {@inheritDoc} */
	@Override
	public boolean canCompose(EObject container, EObject element,
			ICompositionContext context) {
		return findWorkingCompositor(container, element, context) != null;
	}

	/** {@inheritDoc} */
	@Override
	public boolean compose(EObject container, EObject element,
			ICompositionContext context) {
		return findWorkingCompositor(container, element, context).compose(
				container, element, context);
	}

	/** {@inheritDoc} */
	@Override
	public boolean canComposePrototype(Class<? extends EObject> containerClass,
			Prototype prototype) {
		List<ICompositor<EObject>> list = getRegisteredHandlers(containerClass);
		if (list == null) {
			return false;
		}
		for (ICompositor<EObject> compositor : list) {
			if (compositor.canComposePrototype(prototype)) {
				return true;
			}
		}
		return false;
	}

	/**
	 * Returns the first {@link ICompositor} which can compose the given element
	 * (or <code>null</code>).
	 */
	private ICompositor<EObject> findWorkingCompositor(EObject container,
			EObject contained, ICompositionContext context) {
		List<ICompositor<EObject>> list = getRegisteredHandlers(container
				.getClass());
		if (list == null) {
			return null;
		}
		for (ICompositor<EObject> compositor : list) {
			if (compositor.canCompose(container, contained, context)) {
				return compositor;
			}
		}
		return null;
	}

	/** {@inheritDoc} */
	@Override
	protected String getExtensionPointName() {
		return EXTENSION_POINT_NAME;
	}

	/** {@inheritDoc} */
	@Override
	protected String getConfigurationElementName() {
		return CONFIGURATION_ELEMENT_NAME;
	}

	/** {@inheritDoc} */
	@Override
	protected String getHandlerClassAttribute() {
		return HANDLER_CLASS_ATTRIBUTE_NAME;
	}
}
